
#### Script vendredi quanti : Les méthodes d'équations structurelles ####
#########################################################################

# fonction install_library : vérifie si un package est installé avant de le charger
install_library <- function(packages) {
  for (i in packages){
    if(!is.element(i, .packages(all.available = TRUE))) install.packages(i, dep = TRUE)
    library(i,character.only = TRUE)
  }
}


## Packages installer et à charger----
install_library("lavaan")
install_library("semTools")
install_library("semPlot")
install_library("lavaanPlot")
install_library("questionr")
install_library("MVN")
install_library("MIIVsem")
install_library("dplyr")

## Données Political democracy----
data <-PoliticalDemocracy
summary(data)
?PoliticalDemocracy

## Examen de la multinormalité (condition nécessaire)----
mvn(data, univariatePlot = "box")


######## Application 1 : Analyse d'un Modèle de mesure : CFA (analyse factorielle confirmatoire) ##########

## Spécification du modèle CFA (attention aux signes '' au début et à la fin du modèle !!)

model <- '
ind60 =~ x1 + x2 + x3
dem60 =~ y1 + y2 + y3 + y4
dem65 =~ y5 + y6 + y7 + y8

'

## Estimation du modèle CFA
# la commande cfa est adaptée à l'analyse factorielle
fit <- cfa (model, data = data, estimator = "ML") 

## Examen de la solution
# résumé des résultats(avec coeff standardisés et R²)
summary(fit, fit.measures=T, standardized=T, rsquare=T) 

# plus facile de  travailler sur un tableur
clipcopy(standardizedsolution(fit))  

# sélection d'indices d'ajustement du modèle
clipcopy(fitMeasures(fit, c("chisq", "df", "pvalue", "rmsea", "rmsea.ci.lower","rmsea.ci.upper","rmsea.pvalue",
               "cfi", "tli", "srmr", "bic")))  

#indice de fiabilité des échelles de mesure
reliability(fit)


## Dessin (pour un examen rapide)
semPaths(fit, what="mod", whatLabels="std",edge.label.cex=1, style="lisrel",
         layout="tree2" , rotation=2, residuals = T, curvature = 3, nCharNodes = 0, 
         sizeMan=6, structural = F,edge.color="blue")

# Si on souhaite "améliorer" le modèle
modindices(fit) %>% arrange(-mi) %>% head(20)

## Respécification du modèle CFA (à utiliser avec prudence !)
# On sélectionne parmi les propositions les paramètres à "libérer"(pour qu'ils soient estimés)

model2 <- '
ind60 =~ x1 + x2 + x3
dem60 =~ y1 + y2 + y3 + y4
dem65 =~ y5 + y6 + y7 + y8

## covariances ajoutées : à justifier par des arguments théoriques
y1 ~~ y5
y2 ~~ y6
y3 ~~ y7
y4 ~~ y8
'
## Estimation du modèle CFA re-spécifié
fit2 <- cfa (model2, data = data, estimator = "ML")

## Examen de la solution
summary(fit2, fit.measures=T, standardized=T, rsquare=T); clipcopy(standardizedsolution(fit))
clipcopy(fitMeasures(fit2, c("chisq", "df", "pvalue", "rmsea", "rmsea.ci.lower","rmsea.ci.upper",
                             "rmsea.pvalue","cfi", "tli", "srmr", "bic")))

## Dessin du modèle final
semPaths(fit2, what="mod", whatLabels="std",edge.label.cex=1, style="lisrel",
         layout="tree2" , rotation=2, residuals = T, curvature = 2, nCharNodes = 0, 
         sizeMan=6, structural = F,edge.color="blue")

## Un test supplémentaire de comparaison entre les deux modèles : le likehood ratio test
anova(fit, fit2) #test de différence de Khi² 
# Le modèle présentant les valeurs les plus faibles AIC,BIC et Khi² est le mieux ajusté aux données



########### Application 2 : Modélisation simple en piste causale (SEM path modeling) ###########

## Spécification du modèle (on ajoute un modèle de régression au modèle CFA)

model <- '
# variables latentes 
ind60 =~ x1 + x2 + x3
dem60 =~ y1 + y2 + y3 + y4
dem65 =~ y5 + y6 + y7 + y8

# regressions
dem60 ~ ind60
dem65 ~ ind60 + dem60

## covariances ajoutées (voir CFA)
y1 ~~ y5
y2 ~~ y6
y3 ~~ y7
y4 ~~ y8
y6 ~~ y8
y2~~ y4

'

## Estimation du modèle
fit <- sem(model, data = data, estimator = "ML")

## Examen de la solution
summary(fit, fit.measures=T, standardized=T, rsquare=T); clipcopy(standardizedsolution(fit))
clipcopy(fitMeasures(fit, c("chisq", "df", "pvalue", "rmsea", "rmsea.ci.lower","rmsea.ci.upper","cfi", "tli", "srmr")))
modindices(fit) %>% arrange(-mi) %>% head(20)


## Indices de Modification du modèle (à utiliser avec précaution !)
modindices(fit) %>% arrange(-mi) %>% head(20)


## Dessins----
semPaths(fit, what="mod", whatLabels="std",edge.label.cex=1, layout="tree2",style="lisrel",
         rotation=2, residuals = T, curvature = 2, nCharNodes = 0, 
         sizeMan=8, structural = F,edge.color="blue", optimizeLatRes = TRUE)

# Une autre façon de dessiner les modèles
lavaanPlot(model = fit,  node_options = list(shape = "box", fontsize= 14), 
           graph_options = list(overlap = "F", layout="dot"),
           edge_options = list(color = "blue", fontsize = 14),
           coefs = TRUE, sig=0.95, stand=TRUE, covs = TRUE, stars = "regress")


##################### Application 3 Path analysis (variables manifestes) ##############################

data2=bollen1989b
?bollen1989b

mvn(data2, univariatePlot = "box")

## Spécification du modèle
model <-'
unionsen ~  deferenc + laboract + yrsmill
deferenc ~  age
laboract ~  age + deferenc
yrsmill  ~~ age

'

## Estimation du modèle
fit<- sem(model, data=data2, estimator = "ML")

# Examen de la solution
summary(fit, fit.measures=T, standardized=T, rsquare=T); clipcopy(standardizedsolution(fit))
clipcopy(fitMeasures(fit, c("chisq", "df", "pvalue", "rmsea", "rmsea.ci.lower","rmsea.ci.upper","cfi", "tli", "srmr")))


##Dessin
semPaths(fit, what="mod", whatLabels="std",edge.label.cex=1.2, layout="tree2",style="lisrel",
         rotation=2, residuals = T, curvature = 4, nCharNodes = 0, 
         sizeMan=10, edge.color="blue", optimizeLatRes = TRUE)


####### Application 4 : Un peu plus ardu : test des effets médiateurs #######################

### on évalue l'effet mediateur de "laboract" surle lien "yrsmill"/"unionsen" ##

## Spécification du modèle
model <-'
unionsen ~  c*yrsmill
unionsen ~  b*laboract 
laboract ~  a*yrsmill

# analyse de mediation
effet direct := c
effet indirect := a*b
effet total := c+(a*b)
mediation := effet indirect/effet total
'
##Estimation du modèle
fit<- sem(model, data=data2, estimator = "ML")

# Examen de la solution
summary(fit, fit.measures=T, standardized=T, rsquare=T); clipcopy(standardizedsolution(fit))
clipcopy(fitMeasures(fit, c("chisq", "df", "pvalue", "rmsea", "rmsea.ci.lower","rmsea.ci.upper","cfi", "tli", "srmr")))

# Dessin
semPaths(fit, what="mod", whatLabels="std",edge.label.cex=1.8
         , layout="tree2",style="lisrel",
         rotation=1, residuals = T, curvature = 4, nCharNodes = 0, 
         sizeMan=12, edge.color="blue", optimizeLatRes = TRUE)

